﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Xml.Linq;
using Microsoft.Maps.MapControl;
using System.Text;
using System.Windows.Shapes;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using SimpleTalk;

namespace AirportStatus_BingMaps
{
    public partial class MainPage : UserControl
    {
        private Dictionary<Airport, UIElement> uiElements;
        private geoService.GeocodeServiceClient _geoservice;
        private geoService.GeocodeRequest _geocodeRequest;   
        private List<Airport> Airports;
        private MapLayer imageLayer;

        public MainPage()
        {            
            InitializeComponent();
            uiElements = new Dictionary<Airport, UIElement>(); 
            Airports = new List<Airport>();
            GetAirports();

            lstbAirports.ItemsSource = Airports;
        }

        public void GetAirports()
        {
            XDocument doc = XDocument.Load("Airports.xml");

            imageLayer = new MapLayer();
            myMAP.Children.Add(imageLayer);
            _geocodeRequest = new geoService.GeocodeRequest();
            _geocodeRequest.Credentials = new Credentials();
            _geocodeRequest.Credentials.ApplicationId = ((ApplicationIdCredentialsProvider)myMAP.CredentialsProvider).ApplicationId;

            foreach (var item in doc.Elements("Airports").Elements("Airport").ToList())
            {
                var airport = new Airport();
                airport.ID = item.Attributes("id").FirstOrDefault().Value;
                airport.Name = item.Attributes("name").FirstOrDefault().Value;
                Airports.Add(airport);
                Search(airport.ID);
            }
        }

        private void Search(string id)
        {

            if (_geoservice == null)
            {
                _geoservice = new geoService.GeocodeServiceClient("BasicHttpBinding_IGeocodeService");
                _geoservice.GeocodeCompleted += new EventHandler<geoService.GeocodeCompletedEventArgs>(_geoservice_GeocodeCompleted);
            }

            _geocodeRequest.Query = id;            
            _geoservice.GeocodeAsync(_geocodeRequest);

        }

        void _geoservice_GeocodeCompleted(object sender, geoService.GeocodeCompletedEventArgs e)
        {
            if (e.Result.ResponseSummary.StatusCode == geoService.ResponseStatusCode.Success && e.Result.Results[0].EntityType == "Airport")
            {
                Image img = new Image();
                img.Source = new BitmapImage(new Uri("Flag2_Green.png", UriKind.Relative));
                img.Width = 20;
                img.Height = 20;

                imageLayer.AddChild(img, e.Result.Results[0].Locations[0]);

                var airport = Airports.Where(p => p.Name == e.Result.Results[0].DisplayName).FirstOrDefault();
                airport.Location = e.Result.Results[0].Locations[0];

                if (!uiElements.ContainsKey(Airports.Where(p => p.Name == e.Result.Results[0].DisplayName).FirstOrDefault()))
                    uiElements.Add(airport, img);

                GetDescription(airport.ID);
            }
            
        }

        private void GetDescription(string search)
        {
            WebClient client = new WebClient();
            client.DownloadStringCompleted += new DownloadStringCompletedEventHandler(client_DownloadStringCompleted);

            string url = "http://www.airportfact.com/feeds/airport/" + search;
            client.DownloadStringAsync(new Uri(url));
        }

        private void lstbAirports_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            myMAP.SetView((lstbAirports.SelectedItem as Airport).Location, 14);
        }

        private void btnInstall_Click(object sender, RoutedEventArgs e)
        {
            if (Application.Current.InstallState != InstallState.Installed)
                Application.Current.Install();
        }


        private string ReplaceHTMLChars(string str)
        {
            StringBuilder sb = new StringBuilder(str);
            sb.Replace("\n", String.Empty);
            sb.Replace("<b>", String.Empty);
            sb.Replace("</b>", String.Empty);
            sb.Replace("<strong>", String.Empty);
            sb.Replace("</strong>", String.Empty);
            sb.Replace("<small>", String.Empty);
            sb.Replace("</small>", String.Empty);
            sb.Replace("<br />", "\n");
            sb.Replace("<br>", "\n");
            sb.Replace("<p>", "\n");
            sb.Replace("/", "-");
            sb.Replace("</p>", String.Empty);

            return sb.ToString();
        }

        private void client_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                XDocument doc = XDocument.Parse(e.Result);
                var item = doc.Elements("rss").Elements("channel").Elements("item").FirstOrDefault();

                var airport = Airports.Where(p => p.ID == item.Element("title").Value.Substring(0, 3)).FirstOrDefault();
                try
                {
                    ToolTipService.SetToolTip(uiElements[airport], ReplaceHTMLChars(item.Element("description").Value));
                }
                catch (Exception)
                {

                }
            }
        }

    }
}
